;; byte-compiled version will be loaded by bavli.dba

;; TODO
;; better tabbing in report
;; single-key report?
;; sort by masechta order rather than alpha?

(defvar bavli-mblatt [64  157  105  121  22  88  56  40 35  31  32  29  27
       122  112  91  66  49  90  82  119  119  176  113  24
      49  76  14  120  110  142  61  34 34  28  22  4  10  4  73]) 

(defvar bavli-delete-this-record nil)
(defvar bavli-go-back-to-field -1)

(setq dbf-every-change-function 'bavli-check-change-validity)
(setq dbf-enter-field-hook 'bavli-check-special-actions)
(setq db-new-record-function 'bavli-new-entry)

(defun bavli-new-entry (record database)
"Set up a new bavli record defaulting to today and prompting the rest"
(if (>= (database-no-of-records database) 5422) 
 (progn
  (setq bavli-delete-this-record t)
  (read-from-minibuffer "Database full, cannot add new record.  Press ENTER")
;; this next doesn't seem to work
  (db-view-mode))
(progn
 (record-set-field record 'ldate 
	(parse-date-string (calendar-date-string (calendar-current-date))) database)
 (record-set-field record 'bavli 1 database)
 (record-set-field record 'rashi 0 database)
 (record-set-field record 'tosafos 0 database)
 (record-set-field record 'other 0 database)
 (setq bavli-good-value nil)
 (while (equal nil bavli-good-value)
  (progn
   (bavli-masechtot-help)
   (while (not (bavli-valid-masechta (setq bavli-masechta 
      (read-from-minibuffer "Enter Masechta: ")))))
   (switch-to-buffer "bavli.dat")
   (bavli-get-blatt-limits bavli-masechta)
   (while (not (bavli-valid-daf (setq bavli-daf (string-to-int 
      (read-from-minibuffer 
       (concat "Enter Daf (" bavli-min-blatt "-" bavli-max-blatt "): ")
     )))) ))
   (while (not (bavli-valid-amud (setq bavli-amud 
      (read-from-minibuffer "Enter Amud in lowercase, a or b: ")))))
   (if (not (bavli-dupe-check bavli-masechta bavli-daf bavli-amud))
    (setq bavli-good-value t))))
   (kill-buffer "*Masechtot*")
   (switch-to-buffer "bavli.dat")
   (delete-other-windows)
   (record-set-field record 'masechta bavli-masechta database)
   (record-set-field record 'daf bavli-daf database)
   (record-set-field record 'amud bavli-amud database)
)))

(defun bavli-check-change-validity (cfield cold cnew)
;; need a dupe check for daf, amud, masechta plus daf validity
(setq bavli-check-ok t)
(if (eq cfield 'daf)
   (progn
    (bavli-get-blatt-limits (dbf-displayed-record-field 'masechta))
    (if	(not (bavli-valid-daf cnew))
      (setq bavli-check-ok nil))))
(if (or  
	(eq cfield 'daf)
        (eq cfield 'masechta)
        (eq cfield 'amud))
    (progn
	(if (bavli-dupe-check (dbf-displayed-record-field 'masechta)
                               (dbf-displayed-record-field 'daf)
                               (dbf-displayed-record-field 'amud))
	    (setq bavli-check-ok nil))))
(if (equal nil bavli-check-ok)
     (progn
        (setq bavli-go-back-to-field (fieldname->fieldnumber cfield dbc-database))
	(dbf-displayed-record-set-field cfield cold)
        (message "Your entry is invalid, retaining old value.")))
)

(defun bavli-dupe-check (tmasechta tdaf tamud)
(maprecords-macro
 (progn
 (setq bavli-found-dupe nil)
 (progn
   (if (and (equal tdaf (record-field maprecords-record 'daf dbc-database))
            (equal tmasechta (record-field maprecords-record 'masechta dbc-database))
	    (equal tamud (record-field maprecords-record 'amud dbc-database)))
       (progn
	(read-from-minibuffer "Duplicate records not allowed.  Press ENTER.")
	(setq bavli-found-dupe t)
        (maprecords-break))
 ))) dbc-database)
 (if bavli-found-dupe
     t
     nil)
)

(defun bavli-valid-masechta (tmasechta)
  (if (not (= -1 (bavli-find-masechta tmasechta)))
   t
   (progn
    (read-from-minibuffer "Invalid Masechta, press ENTER to try again.")
    nil )
))

(defun bavli-find-masechta (tmasechta)
 (setq bavli-i 0)
 (setq bavli-found -1)
 (while (and (< bavli-i (length bavli-mnames))
	     (= bavli-found -1))
  (progn
    (if (equal (elt bavli-mnames bavli-i) tmasechta)
        (setq bavli-found bavli-i)
	(setq bavli-i (+ 1 bavli-i)))))
 (setq bavli-i bavli-found)
)

(defun bavli-get-blatt-limits (tmasechta)
 (setq bavli-blatt-index (bavli-find-masechta tmasechta))
 (setq bavli-min-blatt 2)
 (setq bavli-max-blatt (aref bavli-mblatt bavli-blatt-index))
;; meilah 2-22 kinnim 23-25 tamid 26-34 midos 35-37
   (cond  
    ((equal tmasechta "Meilah")
      (setq bavli-max-blatt 22))
    ((equal tmasechta "Kinnim")
       (setq bavli-min-blatt 23)
       (setq bavli-max-blatt 25))
    ((equal tmasechta "Tamid")
       (setq bavli-min-blatt 26)
       (setq bavli-max-blatt 34))
    ((equal tmasechta "Midos")
       (setq bavli-min-blatt 35)
       (setq bavli-max-blatt 37)))
)

(defun bavli-valid-daf (tdaf)
;; bavli-get-blatt-limits must have been called first
 (if (and  (integerp tdaf)
           (>= tdaf bavli-min-blatt)
	   (<= tdaf bavli-max-blatt))    
   t
 (progn
  (read-from-minibuffer "Invalid value of Daf for this Masechta - press ENTER.")
    nil))
)

(defun bavli-valid-amud (tamud)
 (if 
       (or (equal tamud "a") 
           (equal tamud "b"))
        t
 (progn
  (read-from-minibuffer "Amud must be 'a' or 'b' (lower case); press ENTER to try again.")
	nil))
)

(defun bavli-check-special-actions ()
;; avoid infinite recursion
 (setq dbf-enter-field-hook nil)
 (if (not (= -1 bavli-go-back-to-field))
  (progn
;; yes, this does work!
   (db-last-field)
   (if (not (= 0 bavli-go-back-to-field))
     (db-next-field bavli-go-back-to-field))
   (setq bavli-go-back-to-field -1)))
 (if (equal t bavli-delete-this-record)
  (progn
   (setq bavli-delete-this-record nil)
   (db-delete-record 1)))
 (setq dbf-enter-field-hook 'bavli-check-special-actions)
)

(defun bavli-masechtot-help ()
 (if (get-buffer "*Masechtot*")
   (kill-buffer "*Masechtot*"))
 (delete-other-windows)
 (split-window)
 (get-buffer "*Masechtot*")
 (switch-to-buffer "*Masechtot*")
 (insert (prin1-to-string bavli-mnames))
 (goto-char (point-min))
 (while (search-forward "\"" nil t)
   (replace-match "" nil t))
 (goto-char (point-min))
 (while (search-forward "(" nil t)
   (replace-match "" nil t))
 (goto-char (point-min))
 (while (search-forward ")" nil t)
   (replace-match "" nil t))
 (goto-char (point-min))
 (while (search-forward " " nil t 8)
   (insert "\n"))
 (goto-char (point-min))
 (insert "Valid Masechtot:\n\n")
)

(defun bavli-report () 
"Give all sorts of stats on learning bavli."
(interactive)
;; We need to build a table of masechtot studied, and we can only do
;; so by scanning the database and keeping track.  We keep other elements
;; for total times studied per masechta, and sum to shas.
 (switch-to-buffer "bavli.dat")
 (db-sort 1)
 (setq bavli-masechtot-learned (make-vector (length bavli-mblatt) nil))
 (setq bavli-bavli-learned (make-vector (length bavli-mblatt) 0))
 (setq bavli-rashi-learned (make-vector (length bavli-mblatt) 0))
 (setq bavli-tosafot-learned (make-vector (length bavli-mblatt) 0))
 (setq bavli-other-learned (make-vector (length bavli-mblatt) 0))
 (setq bavli-masechta-counter 0)
 (setq bavli-amud-learned 0)

 (maprecords-macro
  (progn
   (setq bavli-amud-learned (+ 1 bavli-amud-learned))
;; do we have an entry yet for this record's masechta?   
   (setq bavli-i 0)
   (setq bavli-loc -1)

   (while (and 
             (<= bavli-i bavli-masechta-counter)
	     (= -1 bavli-loc))
    (progn
     (if (equal (aref bavli-masechtot-learned bavli-i) 
              (record-field maprecords-record 'masechta dbc-database))
        (setq bavli-loc bavli-i))
     (setq bavli-i (+ 1 bavli-i))))

   (if (= -1 bavli-loc)
    (progn
     (setq bavli-loc bavli-masechta-counter)
     (aset bavli-masechtot-learned bavli-loc (record-field maprecords-record 'masechta dbc-database))
     (setq bavli-masechta-counter (+ 1 bavli-masechta-counter))))

   (aset bavli-bavli-learned bavli-loc 
     (+ (aref bavli-bavli-learned bavli-loc)
        (record-field maprecords-record 'bavli dbc-database)
     ))
   (aset bavli-rashi-learned bavli-loc
     (+ (aref bavli-rashi-learned bavli-loc) 
        (record-field maprecords-record 'rashi dbc-database)
     ))
   (aset bavli-tosafot-learned bavli-loc
     (+ (aref bavli-tosafot-learned bavli-loc) 
        (record-field maprecords-record 'tosafos dbc-database)
     ))
   (aset bavli-other-learned bavli-loc
     (+ (aref bavli-other-learned bavli-loc) 
        (record-field maprecords-record 'other dbc-database)
     ))
  ) dbc-database)

;; show info
 (if (get-buffer "*Gemara Learning Statistics*")
   (kill-buffer "*Gemara Learning Statistics*"))
 (delete-other-windows)
 (split-window)
 (get-buffer "*Gemara Learning Statistics*")
 (switch-to-buffer "*Gemara Learning Statistics*")
 (delete-other-windows)
 (insert "B\"H\n")
 (insert "           ***** Gemara Bavli Learnings Statistics *****\n\n")
 (insert "You have made entries for " (int-to-string bavli-amud-learned) " amud")
 (insert " in " (int-to-string bavli-masechta-counter) " different masechtot.\n\n")
 (insert "Total times amudim learned, by masechta:\n\n")
 (insert "                    Gemara\tRashi\tTosafos\tOther\n\n")
 (setq bavli-i 0)
 (setq bavli-bavli-total 0)
 (setq bavli-rashi-total 0)
 (setq bavli-tosafot-total 0)
 (setq bavli-other-total 0)
 (while (< bavli-i bavli-masechta-counter)
  (progn
   (insert (format "%-20s" (aref bavli-masechtot-learned bavli-i) ))
   (insert (int-to-string (aref bavli-bavli-learned bavli-i)) "\t\t")
   (insert (int-to-string (aref bavli-rashi-learned bavli-i)) "\t")
   (insert (int-to-string (aref bavli-tosafot-learned bavli-i)) "\t")
   (insert (int-to-string (aref bavli-other-learned bavli-i)) "\t")
   (insert "\n")
   (setq bavli-bavli-total (+ bavli-bavli-total (aref bavli-bavli-learned bavli-i)))
   (setq bavli-rashi-total (+ bavli-rashi-total (aref bavli-rashi-learned bavli-i)))
   (setq bavli-tosafot-total (+ bavli-tosafot-total (aref bavli-tosafot-learned bavli-i)))
   (setq bavli-other-total (+ bavli-other-total (aref bavli-other-learned bavli-i)))
 (setq bavli-i (+ 1 bavli-i)) ))

;; now print summary stats
 (insert "\nYou have learned Gemara " (int-to-string bavli-bavli-total) " times,\n")
 (insert   "                  Rashi " (int-to-string bavli-rashi-total) " times,\n")
 (insert   "                Tosefot " (int-to-string bavli-tosafot-total) " times,\n")
 (insert   "      and other sources " (int-to-string bavli-other-total) " times.\n")  

)

(defun bavli-sort ()
"Sort the database by tractate ordering."
 (interactive)
 (save-excursion
  (switch-to-buffer "bavli.dat")
  (database-sort dbc-database 'bavli-tractate-sort)
  (dbf-finished-sorting))
)
